using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using WSv2.DN35.CS_Sample_Code.MKDenialWebService;

namespace WSv2.DN35.CS_Sample_Code {
    public partial class Form1 : Form {
        public Form1() {
            InitializeComponent();
        }

        #region support methods - not code samples per se
        /// <summary>
        /// Dump information regarding an object into a string for display
        /// Borrowed from http://www.developer.com/net/csharp/article.php/3713886
        /// </summary>
        /// <param name="o">An object</param>
        /// <returns>Details about the object as a string</returns>
        public static string DisplayObjectInfo(Object o) {
            StringBuilder sb = new StringBuilder();

            // Include the type of the object
            System.Type type = o.GetType();
            sb.Append("Type: " + type.Name);

            // Include information for each Field
            sb.Append("\r\n\r\nFields:");
            System.Reflection.FieldInfo[] fi = type.GetFields();
            if (fi.Length > 0) {
                foreach (FieldInfo f in fi) {
                    sb.Append("\r\n " + f.ToString() + " = " +
                              f.GetValue(o));
                }
            } else
                sb.Append("\r\n None");

            // Include information for each Property
            sb.Append("\r\n\r\nProperties:");
            System.Reflection.PropertyInfo[] pi = type.GetProperties();
            if (pi.Length > 0) {
                foreach (PropertyInfo p in pi) {
                    sb.Append("\r\n " + p.ToString() + " = " +
                              p.GetValue(o, null));
                }
            } else
                sb.Append("\r\n None");

            return sb.ToString();
        }
        #endregion

        private void loginButton_Click(object sender, EventArgs e) {
            // so I don't hard code my username/password into the demo
            // I've put in fields for this. Simply use the username/password
            // given to you by MKDS
            if (usernameTextBox.Text.Equals(string.Empty)) {
                MessageBox.Show("Username is required.");
                usernameTextBox.Select();
            } else if (passwordTextBox.Text.Equals(string.Empty)) {
                MessageBox.Show("Password is required.");
                passwordTextBox.Select();
            } else {
                // Here I do a simple call for user details to make sure the
                // auth information is correct. If it isn't correct, or the
                // account isn't set up for web services, this will throw
                // a fault exception.
                try {
                    DPLv2Client ws = new DPLv2Client();
                    DPLv2Credentials auth = new DPLv2Credentials();
                    auth.Username = usernameTextBox.Text;
                    auth.Password = passwordTextBox.Text;
                    DPLv2User user = ws.GetUserDetails(auth);
                    if (user.username == usernameTextBox.Text) {
                        mainTabControl.Enabled = true;
                        MessageBox.Show("Success");
                    }
                }
                catch (Exception ex) {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void lastUpdateInvokeButton_Click(object sender, EventArgs e) {
            // A simple enough request for the last update to the list
            // this can be used to help automate download of files from
            // the downloads section, if that is something you require.
            // It should be noted that the best time to check this is after
            // 8pm EST if that is what you are trying to do, as before then 
            // another update may overwrite the files.
            DPLv2Client ws = new DPLv2Client();
            DPLv2Credentials auth = new DPLv2Credentials();
            auth.Username = usernameTextBox.Text;
            auth.Password = passwordTextBox.Text;
            DateTime lastUpdate = ws.LastUpdate(auth);
            lastUpdateTextBox.Text = "Last Update: " + lastUpdate.ToString("d");
        }

        private void userDetailsInvokeButton_Click(object sender, EventArgs e) {
            // This call will give you some simple details about the account
            // used to log in
            DPLv2Client ws = new DPLv2Client();
            DPLv2Credentials auth = new DPLv2Credentials();
            auth.Username = usernameTextBox.Text;
            auth.Password = passwordTextBox.Text;
            DPLv2User user = ws.GetUserDetails(auth);
            userDetailsTextBox.Text = DisplayObjectInfo(user);
        }

        private void searchSimpleInvokeButton_Click(object sender, EventArgs e) {
            // this represents a very simple request from the site
            DPLv2Client ws = new DPLv2Client();
            DPLv2Credentials auth = new DPLv2Credentials();
            auth.Username = usernameTextBox.Text;
            auth.Password = passwordTextBox.Text;
            // all searches are done via this Search object, we'll call ours "request"
            Search request = new Search();
            // this will ensure the country "XX" is included if a country match construct
            // is in the search request somewhere
            request.AlwaysIncludeUnknownCountry = true;
            // this is the logic that combines the groups, like Group1 AND Group2...
            request.Connect = ConnectType.and;
            // If your account uses a "Common Word List" this will determine whether we
            // ignore those common words in your search
            request.ExcludeCommonWords = true;
            // this is a very simple match, and not recommended
            // we're matching any entry that has the EXACT name "ACECHILLY"
            // Generally this doesn't work well for names, as there is usually
            // more than one way to enter a name
            Match m = new Match();
            // The field is what we are searching on. Similar to the site, except
            // here we've combined STREET1 and STREET2 into one field for ease
            // of searching
            m.Field = FieldType.Name;
            // We need a keyword
            m.Keyword = "ACECHILLY";
            // in this case the Level isn't used, but if it isn't specified, a Fault
            // will be thrown
            m.Level = 1;
            // We are matching on the entire phrase (the keyword above)
            m.Scope = MatchScope.Phrase;
            // and we are matching if it "IS" the same as an item on the list
            m.Type = MatchType.Is;
            Group g = new Group();
            // similar to the above connect type on the search construct, but this affects how matches
            // in this group are connected to each other
            g.Connect = ConnectType.and;
            // a group consists of multiple matches, here we're just using 1
            g.Matches = new Match[1] { m };
            // and those groups are then bundled together on the Search request object, here
            // we just have 1
            request.Groups = new Group[1] { g };
            // we subject the Search request object to the service
            DPLv2Report searchResults = ws.SearchDpl(auth, request);
            // and then view the results. Here we're binding them to a DataGridView
            // we could also simply check searchResults.Hits.Count to see how many
            // hits were returned, and if 0 there is no need to worry
            searchSimpleDataGridView.DataSource = searchResults.Hits;
        }

        private void searchTypicalButton_Click(object sender, EventArgs e) {
            // Here we're going to try something a bit more typical
            DPLv2Client ws = new DPLv2Client();
            DPLv2Credentials auth = new DPLv2Credentials();
            auth.Username = usernameTextBox.Text;
            auth.Password = passwordTextBox.Text;
            Search request = new Search();
            request.AlwaysIncludeUnknownCountry = true;
            request.Connect = ConnectType.and;
            request.ExcludeCommonWords = true;
            // Match any DPL entry where 2 words in the entry sound like 2 of our keywords
            Match m = new Match();
            m.Field = FieldType.Name;
            m.Keyword = "Anderson, Robert";
            // level represents how many words much match for the match to be considered a hit
            // when the Scope is set to Word
            m.Level = 2;
            m.Scope = MatchScope.Word;
            // The IsSimilarTo Type uses the Metaphone phonetic search to constitute a hit
            m.Type = MatchType.IsSimilarTo;
            // Also require a match on country
            Match cm = new Match();
            cm.Field = FieldType.Country;
            cm.Keyword = "US";
            cm.Level = 1;
            cm.Scope = MatchScope.Phrase;
            cm.Type = MatchType.Is;
            Group g = new Group();
            g.Connect = ConnectType.and;
            g.Matches = new Match[2] { m, cm };
            request.Groups = new Group[1] { g };
            DPLv2Report searchResults = ws.SearchDpl(auth, request);
            searchTypicalDataGridView.DataSource = searchResults.Hits;
        }

        private void searchComplexButton_Click(object sender, EventArgs e) {
            // And now for something really complex.
            DPLv2Client ws = new DPLv2Client();
            DPLv2Credentials auth = new DPLv2Credentials();
            auth.Username = usernameTextBox.Text;
            auth.Password = passwordTextBox.Text;
            Search request = new Search();
            request.AlwaysIncludeUnknownCountry = true;
            // we're going to require all the groups on this one
            request.Connect = ConnectType.and;
            request.ExcludeCommonWords = true;
            // the first group will be names. Perhaps we have the following names
            // and we are looking for a hit on any of them
            Match m1 = new Match();
            m1.Field = FieldType.Name;
            m1.Keyword = "Reza Amiri";
            m1.Level = 2;
            m1.Scope = MatchScope.Word;
            m1.Type = MatchType.Is;
            Match m2 = new Match();
            m2.Field = FieldType.Name;
            m2.Keyword = "Ray Amiri";
            m2.Level = 2;
            m2.Scope = MatchScope.Word;
            m2.Type = MatchType.Is;
            Match m3 = new Match();
            m3.Field = FieldType.Name;
            m3.Keyword = "RACC";
            m3.Level = 2;
            m3.Scope = MatchScope.Word;
            m3.Type = MatchType.Is;
            // our second group might be countries, we are only concerned with
            // US (USA), IR (Iran), or XX (unspecified) countries
            Group g1 = new Group();
            g1.Connect = ConnectType.or;
            g1.Matches = new Match[] { m1, m2, m3 };
            Match m4 = new Match();
            m4.Field = FieldType.Country;
            m4.Keyword = "US";
            m4.Level = 1;
            m4.Scope = MatchScope.Phrase;
            m4.Type = MatchType.Is;
            Match m5 = new Match();
            m5.Field = FieldType.Country;
            m5.Keyword = "IR";
            m5.Level = 1;
            m5.Scope = MatchScope.Phrase;
            m5.Type = MatchType.Is;
            Match m6 = new Match();
            m6.Field = FieldType.Country;
            m6.Keyword = "XX";
            m6.Level = 1;
            m6.Scope = MatchScope.Phrase;
            m6.Type = MatchType.Is;
            Group g2 = new Group();
            g2.Connect = ConnectType.or;
            g2.Matches = new Match[] { m4, m5, m6 };
            // and our third group, which is added here on-the-fly, is a special group
            // that restricts the matching to just list codes that are published by the US
            // perhaps we aren't concerned about lists outside the US
            request.Groups = new Group[] { g1, g2, ws.GroupFactory(auth, GroupFilterType.RestrictToCodesBySourceCountry, "US") };
            DPLv2Report searchResults = ws.SearchDpl(auth, request);
            searchComplexDataGridView.DataSource = searchResults.Hits;
        }

        private void aliasesInvokeButton_Click(object sender, EventArgs e) {
            // And once we have some data, like in the complex search, we might want more
            // data about possible aliases used by the individual. Here you can find out
            // what other DPL entries are related.
            DPLv2Client ws = new DPLv2Client();
            DPLv2Credentials auth = new DPLv2Credentials();
            auth.Username = usernameTextBox.Text;
            auth.Password = passwordTextBox.Text;
            DPLv2Entry[] akas = ws.GetAliases(auth, 157);
            aliasesDataGridView.DataSource = akas;
        }

    }
}